<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Listens for Instant Payment Notification from iPay
 *
 * This script waits for Payment notification from iPay,
 * then double checks that data by sending it back to iPay.
 * If iPay verifies this then it sets up the enrolment for that
 * user.
 *
 * @package    enrol_ipay
 * @copyright iPay Limited
 * @author     SlashDotLabs <projects@slashdotlabs.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

// Disable moodle specific debug messages and any errors in output,
// comment out when debugging or better look into error log!
define('NO_DEBUG_DISPLAY', true);

// @codingStandardsIgnoreLine This script does not require login.
require "../../config.php";
require_once "lib.php";
require_once $CFG->libdir . '/enrollib.php';
require_once $CFG->libdir . '/filelib.php';

// Keep out casual intruders
if (empty($_GET)) {
	http_response_code(400);
	throw new moodle_exception('invalidrequest', 'core_error');
}

// iPay does not like when we return error messages here,
// the custom handler just logs exceptions and stops.
set_exception_handler(\enrol_ipay\util::get_exception_handler());

// Make sure we are enabled in the first place.
if (!enrol_is_enabled('ipay')) {
	http_response_code(503);
	throw new moodle_exception('errdisabled', 'enrol_ipay');
}

/// Read all the data from iPay and get it ready for later;
/// we expect only valid UTF-8 encoding, it is the responsibility
/// of user to set it up properly in iPay business account,
/// it is documented in docs wiki.

$req = 'cmd=_notify-validate';

$data = new stdClass();

$plugin = enrol_get_plugin('ipay');

$vendor = $plugin->get_config('vendorid'); // the vendor ID
$txnid = $_GET['id'];
$ivm = $_GET['ivm'];
$qwh = $_GET['qwh'];
$afd = $_GET['afd'];
$poi = $_GET['poi'];
$uyt = $_GET['uyt'];
$ifd = $_GET['ifd'];
$amount = $_GET['mc'];

$ipn_ke = "https://www.ipayafrica.com/ipn";
$ipn_tz = "https://payments.elipa.co.tz/v3/tz/ipn";
$ipn_ug = "https://payments.elipa.co.ug/v3/ug/ipn";
$ipn_tg = "https://payments.elipa.tg/v1/tg/index.php/ipn/check";

$ipn_base = "";

switch ($plugin->get_config('country')) {
case 'ke':
	$ipn_base = $ipn_ke;
	break;

case 'tz':
	$ipn_base = $ipn_tz;
	break;

case 'ug':
	$ipn_base = $ipn_ug;
	break;

case 'tg':
	$ipn_base = $ipn_tg;
	break;

default:
	echo "Unknown country of operation";
	exit();
	break;
}

$ipnurl = $ipn_base . "?vendor=" . $vendor . "&id=" . $txnid . "&ivm=" . $ivm . "&qwh=" . $qwh . "&afd=" . $afd . "&poi=" . $poi . "&uyt=" . $uyt . "&ifd=" . $ifd;

print_r($ipnurl);

if ($plugin->get_config('country')) {
	$response = "aei7p7yrx4ae34";
} else {
	$fp = fopen($ipnurl, "rb");
	$response = stream_get_contents($fp, -1, -1);
	fclose($fp);
}

// print_r($response);
// exit();

// foreach ($_POST as $key => $value) {
// 	if ($key !== clean_param($key, PARAM_ALPHANUMEXT)) {
// 		throw new moodle_exception('invalidrequest', 'core_error', '', null, $key);
// 	}
// 	if (is_array($value)) {
// 		throw new moodle_exception('invalidrequest', 'core_error', '', null, 'Unexpected array param: ' . $key);
// 	}
// 	$req .= "&$key=" . urlencode($value);
// 	$data->$key = fix_utf8($value);
// }

if (empty($txnid)) {
	throw new moodle_exception('invalidrequest', 'core_error', '', null, 'Missing request param: id');
}

$custom = explode('-', $txnid);

if (empty($custom) || count($custom) < 3) {
	throw new moodle_exception('invalidrequest', 'core_error', '', null, 'Invalid value of the request param: custom');
}

$data->userid = (int) $custom[0];
$data->courseid = (int) $custom[1];
$data->instanceid = (int) $custom[2];
$data->payment_gross = $amount;
$data->payment_currency = $plugin->get_config('currency');
$data->txn_id = $txnid;
$data->timeupdated = time();

$user = $DB->get_record("user", array("id" => $data->userid), "*", MUST_EXIST);
$course = $DB->get_record("course", array("id" => $data->courseid), "*", MUST_EXIST);
$context = context_course::instance($course->id, MUST_EXIST);

$PAGE->set_context($context);

$plugin_instance = $DB->get_record("enrol", array("id" => $data->instanceid, "enrol" => "ipay", "status" => 0), "*", MUST_EXIST);
$plugin = enrol_get_plugin('ipay');

switch ($response) {
case 'aei7p7yrx4ae34':
// Success
case 'eq3i7p5yt7645e':
	// Overpaid
	if (!$user = $DB->get_record('user', array('id' => $data->userid))) {
		// Check that user exists
		\enrol_ipay\util::message_ipay_error_to_admin("User $data->userid doesn't exist", $data);
		die;
	}

	if (!$course = $DB->get_record('course', array('id' => $data->courseid))) {
		// Check that course exists
		\enrol_ipay\util::message_ipay_error_to_admin("Course $data->courseid doesn't exist", $data);
		die;
	}

	$coursecontext = context_course::instance($course->id, IGNORE_MISSING);

	// Check that amount paid is the correct amount
	if ((float) $plugin_instance->cost <= 0) {
		$cost = (float) $plugin->get_config('cost');
	} else {
		$cost = (float) $plugin_instance->cost;
	}

	// Use the same rounding of floats as on the enrol form.
	$cost = format_float($cost, 2, false);

	if ($data->payment_gross < $cost) {
		\enrol_ipay\util::message_ipay_error_to_admin("Amount paid is not enough ($data->payment_gross < $cost))", $data);
		die;

	}
	// Use the queried course's full name for the item_name field.
	$data->item_name = $course->fullname;

	// ALL CLEAR !

	$DB->insert_record("enrol_ipay", $data);

	if ($plugin_instance->enrolperiod) {
		$timestart = time();
		$timeend = $timestart + $plugin_instance->enrolperiod;
	} else {
		$timestart = 0;
		$timeend = 0;
	}

	// Enrol user
	$plugin->enrol_user($plugin_instance, $user->id, $plugin_instance->roleid, $timestart, $timeend);

	// Pass $view=true to filter hidden caps if the user cannot see them
	if ($users = get_users_by_capability($context, 'moodle/course:update', 'u.*', 'u.id ASC',
		'', '', '', '', false, true)) {
		$users = sort_by_roleassignment_authority($users, $context);
		$teacher = array_shift($users);
	} else {
		$teacher = false;
	}

	$mailstudents = $plugin->get_config('mailstudents');
	$mailteachers = $plugin->get_config('mailteachers');
	$mailadmins = $plugin->get_config('mailadmins');
	$shortname = format_string($course->shortname, true, array('context' => $context));

	if (!empty($mailstudents)) {
		$a = new stdClass();
		$a->coursename = format_string($course->fullname, true, array('context' => $coursecontext));
		$a->profileurl = "$CFG->wwwroot/user/view.php?id=$user->id";

		$eventdata = new \core\message\message();
		$eventdata->courseid = $course->id;
		$eventdata->modulename = 'moodle';
		$eventdata->component = 'enrol_ipay';
		$eventdata->name = 'ipay_enrolment';
		$eventdata->userfrom = empty($teacher) ? core_user::get_noreply_user() : $teacher;
		$eventdata->userto = $user;
		$eventdata->subject = get_string("enrolmentnew", 'enrol', $shortname);
		$eventdata->fullmessage = get_string('welcometocoursetext', '', $a);
		$eventdata->fullmessageformat = FORMAT_PLAIN;
		$eventdata->fullmessagehtml = '';
		$eventdata->smallmessage = '';
		message_send($eventdata);

	}

	if (!empty($mailteachers) && !empty($teacher)) {
		$a->course = format_string($course->fullname, true, array('context' => $coursecontext));
		$a->user = fullname($user);

		$eventdata = new \core\message\message();
		$eventdata->courseid = $course->id;
		$eventdata->modulename = 'moodle';
		$eventdata->component = 'enrol_ipay';
		$eventdata->name = 'ipay_enrolment';
		$eventdata->userfrom = $user;
		$eventdata->userto = $teacher;
		$eventdata->subject = get_string("enrolmentnew", 'enrol', $shortname);
		$eventdata->fullmessage = get_string('enrolmentnewuser', 'enrol', $a);
		$eventdata->fullmessageformat = FORMAT_PLAIN;
		$eventdata->fullmessagehtml = '';
		$eventdata->smallmessage = '';
		message_send($eventdata);
	}

	if (!empty($mailadmins)) {
		$a->course = format_string($course->fullname, true, array('context' => $coursecontext));
		$a->user = fullname($user);
		$admins = get_admins();
		foreach ($admins as $admin) {
			$eventdata = new \core\message\message();
			$eventdata->courseid = $course->id;
			$eventdata->modulename = 'moodle';
			$eventdata->component = 'enrol_ipay';
			$eventdata->name = 'ipay_enrolment';
			$eventdata->userfrom = $user;
			$eventdata->userto = $admin;
			$eventdata->subject = get_string("enrolmentnew", 'enrol', $shortname);
			$eventdata->fullmessage = get_string('enrolmentnewuser', 'enrol', $a);
			$eventdata->fullmessageformat = FORMAT_PLAIN;
			$eventdata->fullmessagehtml = '';
			$eventdata->smallmessage = '';
			message_send($eventdata);
		}
	}
	print_r($response);
	$destination = "$CFG->wwwroot/course/view.php?id=$data->courseid";
	redirect($destination, get_string('paymentthanks', '', $fullname));
	break;
case 'fe2707etr5s4wq':
	// Failed
	$plugin->unenrol_user($plugin_instance, $data->userid);
	\enrol_ipay\util::message_ipay_error_to_admin("Status not completed or . User unenrolled from course",
		$data);
	die;

case 'dtfi4p7yty45wq':
	// Underpaid
	$plugin->unenrol_user($plugin_instance, $data->userid);
	\enrol_ipay\util::message_ipay_error_to_admin("Transaction underpaid. User unenrolled from course",
		$data);
	die;
case 'cr5i3pgy9867e1':
	// Used
	$plugin->unenrol_user($plugin_instance, $data->userid);
	\enrol_ipay\util::message_ipay_error_to_admin("Payment is already used. User unenrolled from course",
		$data);
	die;
case 'bdi6p2yy76etrs':
	// Pending
	$plugin->unenrol_user($plugin_instance, $data->userid);
	\enrol_ipay\util::message_ipay_error_to_admin("Status is pending. User unenrolled from course",
		$data);
	die;

default:
	$plugin->unenrol_user($plugin_instance, $data->userid);
	\enrol_ipay\util::message_ipay_error_to_admin("Unknown payment status. User unenrolled from course",
		$data);
	die;
}
